@@ -3,8 +3,8 @@ import inspect
 import os
 import sys
 
+from . import __version__
 from . import command
-from . import package_dir
 from . import util
 from .util import compat
 from .util.compat import SafeConfigParser
@@ -12,7 +12,7 @@ from .util.compat import SafeConfigParser
 
 class Config(object):
 
-    """Represent an Alembic configuration.
+    r"""Represent an Alembic configuration.
 
     Within an ``env.py`` script, this is available
     via the :attr:`.EnvironmentContext.config` attribute,
@@ -40,7 +40,7 @@ class Config(object):
         from alembic.config import Config
         alembic_cfg = Config()
         alembic_cfg.set_main_option("script_location", "myapp:migrations")
-        alembic_cfg.set_main_option("url", "postgresql://foo/bar")
+        alembic_cfg.set_main_option("sqlalchemy.url", "postgresql://foo/bar")
         alembic_cfg.set_section_option("mysection", "foo", "bar")
 
     .. warning::
@@ -58,7 +58,7 @@ class Config(object):
             alembic_cfg.attributes['connection'] = connection
             command.upgrade(alembic_cfg, "head")
 
-    :param file_: name of the .ini file to open.
+    :param file\_: name of the .ini file to open.
     :param ini_section: name of the main Alembic section within the
      .ini file
     :param output_buffer: optional file-like input buffer which
@@ -67,8 +67,6 @@ class Config(object):
     :param stdout: buffer where the "print" output of commands will be sent.
      Defaults to ``sys.stdout``.
 
-     .. versionadded:: 0.4
-
     :param config_args: A dictionary of keys and values that will be used
      for substitution in the alembic config file.  The dictionary as given
      is **copied** to a new one, stored locally as the attribute
@@ -77,13 +75,9 @@ class Config(object):
      dictionary before the dictionary is passed to ``SafeConfigParser()``
      to parse the .ini file.
 
-     .. versionadded:: 0.7.0
-
     :param attributes: optional dictionary of arbitrary Python keys/values,
      which will be populated into the :attr:`.Config.attributes` dictionary.
 
-     .. versionadded:: 0.7.5
-
      .. seealso::
 
         :ref:`connection_sharing`
@@ -100,9 +94,7 @@ class Config(object):
         config_args=util.immutabledict(),
         attributes=None,
     ):
-        """Construct a new :class:`.Config`
-
-        """
+        """Construct a new :class:`.Config`"""
         self.config_file_name = file_
         self.config_ini_section = ini_section
         self.output_buffer = output_buffer
@@ -118,8 +110,6 @@ class Config(object):
     Within an ``env.py`` script this can be accessed via the
     :attr:`.EnvironmentContext.config` attribute.
 
-    .. versionadded:: 0.6.0
-
     .. seealso::
 
         :meth:`.EnvironmentContext.get_x_argument`
@@ -133,7 +123,7 @@ class Config(object):
     """Name of the config file section to read basic configuration
     from.  Defaults to ``alembic``, that is the ``[alembic]`` section
     of the .ini file.  This value is modified using the ``-n/--name``
-    option to the Alembic runnier.
+    option to the Alembic runner.
 
     """
 
@@ -148,8 +138,6 @@ class Config(object):
         a :class:`sqlalchemy.engine.base.Connection` when calling
         commands from :mod:`alembic.command` programmatically.
 
-        .. versionadded:: 0.7.5
-
         .. seealso::
 
             :ref:`connection_sharing`
@@ -210,13 +198,19 @@ class Config(object):
         commands.
 
         """
+        import alembic
+
+        package_dir = os.path.abspath(os.path.dirname(alembic.__file__))
         return os.path.join(package_dir, "templates")
 
-    def get_section(self, name):
+    def get_section(self, name, default=None):
         """Return all the configuration options from a given .ini file section
         as a dictionary.
 
         """
+        if not self.file_config.has_section(name):
+            return default
+
         return dict(self.file_config.items(name))
 
     def set_main_option(self, name, value):
@@ -264,9 +258,7 @@ class Config(object):
         self.file_config.set(section, name, value)
 
     def get_section_option(self, section, name, default=None):
-        """Return an option from the given section of the .ini file.
-
-        """
+        """Return an option from the given section of the .ini file."""
         if not self.file_config.has_section(section):
             raise util.CommandError(
                 "No config file %r found, or file has no "
@@ -293,7 +285,7 @@ class CommandLine(object):
         self._generate_args(prog)
 
     def _generate_args(self, prog):
-        def add_options(parser, positional, kwargs):
+        def add_options(fn, parser, positional, kwargs):
             kwargs_opts = {
                 "template": (
                     "-t",
@@ -397,14 +389,6 @@ class CommandLine(object):
                         "of database to model.",
                     ),
                 ),
-                "head_only": (
-                    "--head-only",
-                    dict(
-                        action="store_true",
-                        help="Deprecated.  Use --verbose for "
-                        "additional output",
-                    ),
-                ),
                 "rev_range": (
                     "-r",
                     "--rev-range",
@@ -422,6 +406,22 @@ class CommandLine(object):
                         help="Indicate the current revision",
                     ),
                 ),
+                "purge": (
+                    "--purge",
+                    dict(
+                        action="store_true",
+                        help="Unconditionally erase the version table "
+                        "before stamping",
+                    ),
+                ),
+                "package": (
+                    "--package",
+                    dict(
+                        action="store_true",
+                        help="Write empty __init__.py files to the "
+                        "environment and version locations",
+                    ),
+                ),
             }
             positional_help = {
                 "directory": "location of scripts directory",
@@ -435,20 +435,31 @@ class CommandLine(object):
                     parser.add_argument(*args, **kw)
 
             for arg in positional:
-                if arg == "revisions":
+                if (
+                    arg == "revisions"
+                    or fn in positional_translations
+                    and positional_translations[fn][arg] == "revisions"
+                ):
                     subparser.add_argument(
-                        arg, nargs="+", help=positional_help.get(arg)
+                        "revisions",
+                        nargs="+",
+                        help=positional_help.get("revisions"),
                     )
                 else:
                     subparser.add_argument(arg, help=positional_help.get(arg))
 
         parser = ArgumentParser(prog=prog)
+
+        parser.add_argument(
+            "--version", action="version", version="%%(prog)s %s" % __version__
+        )
         parser.add_argument(
             "-c",
             "--config",
             type=str,
-            default="alembic.ini",
-            help="Alternate config file",
+            default=os.environ.get("ALEMBIC_CONFIG", "alembic.ini"),
+            help="Alternate config file; defaults to value of "
+            'ALEMBIC_CONFIG environment variable, or "alembic.ini"',
         )
         parser.add_argument(
             "-n",
@@ -471,6 +482,8 @@ class CommandLine(object):
         )
         subparsers = parser.add_subparsers()
 
+        positional_translations = {command.stamp: {"revision": "revisions"}}
+
         for fn in [getattr(command, n) for n in dir(command)]:
             if (
                 inspect.isfunction(fn)
@@ -486,8 +499,27 @@ class CommandLine(object):
                     positional = spec[0][1:]
                     kwarg = []
 
-                subparser = subparsers.add_parser(fn.__name__, help=fn.__doc__)
-                add_options(subparser, positional, kwarg)
+                if fn in positional_translations:
+                    positional = [
+                        positional_translations[fn].get(name, name)
+                        for name in positional
+                    ]
+
+                # parse first line(s) of helptext without a line break
+                help_ = fn.__doc__
+                if help_:
+                    help_text = []
+                    for line in help_.split("\n"):
+                        if not line.strip():
+                            break
+                        else:
+                            help_text.append(line.strip())
+                else:
+                    help_text = ""
+                subparser = subparsers.add_parser(
+                    fn.__name__, help=" ".join(help_text)
+                )
+                add_options(fn, subparser, positional, kwarg)
                 subparser.set_defaults(cmd=(fn, positional, kwarg))
         self.parser = parser
 
