@@ -1,8 +1,13 @@
 from sqlalchemy import schema as sa_schema
+from sqlalchemy.sql.schema import Column
+from sqlalchemy.sql.schema import Constraint
+from sqlalchemy.sql.schema import Index
 from sqlalchemy.types import Integer
 from sqlalchemy.types import NULLTYPE
 
 from .. import util
+from ..util import sqla_compat
+from ..util.compat import raise_
 from ..util.compat import string_types
 
 
@@ -15,7 +20,6 @@ class SchemaObjects(object):
         columns = [sa_schema.Column(n, NULLTYPE) for n in cols]
         t = sa_schema.Table(table_name, m, *columns, schema=schema)
         p = sa_schema.PrimaryKeyConstraint(*[t.c[n] for n in cols], name=name)
-        t.append_constraint(p)
         return p
 
     def foreign_key_constraint(
@@ -113,10 +117,13 @@ class SchemaObjects(object):
         }
         try:
             const = types[type_]
-        except KeyError:
-            raise TypeError(
-                "'type' can be one of %s"
-                % ", ".join(sorted(repr(x) for x in types))
+        except KeyError as ke:
+            raise_(
+                TypeError(
+                    "'type' can be one of %s"
+                    % ", ".join(sorted(repr(x) for x in types))
+                ),
+                from_=ke,
             )
         else:
             const = const(name=name)
@@ -136,7 +143,36 @@ class SchemaObjects(object):
 
     def table(self, name, *columns, **kw):
         m = self.metadata()
-        t = sa_schema.Table(name, m, *columns, **kw)
+
+        cols = [
+            sqla_compat._copy(c) if c.table is not None else c
+            for c in columns
+            if isinstance(c, Column)
+        ]
+        # these flags have already added their UniqueConstraint /
+        # Index objects to the table, so flip them off here.
+        # SQLAlchemy tometadata() avoids this instead by preserving the
+        # flags and skipping the constraints that have _type_bound on them,
+        # but for a migration we'd rather list out the constraints
+        # explicitly.
+        _constraints_included = kw.pop("_constraints_included", False)
+        if _constraints_included:
+            for c in cols:
+                c.unique = c.index = False
+
+        t = sa_schema.Table(name, m, *cols, **kw)
+
+        constraints = [
+            sqla_compat._copy(elem, target_table=t)
+            if getattr(elem, "parent", None) is not None
+            else elem
+            for elem in columns
+            if isinstance(elem, (Constraint, Index))
+        ]
+
+        for const in constraints:
+            t.append_constraint(const)
+
         for f in t.foreign_keys:
             self._ensure_table_for_fk(m, f)
         return t
@@ -146,8 +182,11 @@ class SchemaObjects(object):
 
     def index(self, name, tablename, columns, schema=None, **kw):
         t = sa_schema.Table(
-            tablename or "no_table", self.metadata(), schema=schema
+            tablename or "no_table",
+            self.metadata(),
+            schema=schema,
         )
+        kw["_table"] = t
         idx = sa_schema.Index(
             name,
             *[util.sqla_compat._textual_index_column(t, n) for n in columns],
