@@ -4,9 +4,10 @@ import os
 import shutil
 import textwrap
 
-from sqlalchemy.testing import engines
+from sqlalchemy.testing import config
 from sqlalchemy.testing import provision
 
+from . import util as testing_util
 from .. import util
 from ..script import Script
 from ..script import ScriptDirectory
@@ -27,9 +28,12 @@ def staging_env(create=True, template="generic", sourceless=False):
 
     cfg = _testing_config()
     if create:
+
         path = os.path.join(_get_staging_directory(), "scripts")
-        if os.path.exists(path):
-            shutil.rmtree(path)
+        assert not os.path.exists(path), (
+            "staging directory %s already exists; poor cleanup?" % path
+        )
+
         command.init(cfg, path, template=template)
         if sourceless:
             try:
@@ -57,6 +61,9 @@ def staging_env(create=True, template="generic", sourceless=False):
 
 
 def clear_staging_env():
+    from sqlalchemy.testing import engines
+
+    engines.testing_reaper.close_all()
     shutil.rmtree(_get_staging_directory(), True)
 
 
@@ -87,25 +94,28 @@ config = context.config
         f.write(txt)
 
 
-def _sqlite_file_db(tempname="foo.db"):
+def _sqlite_file_db(tempname="foo.db", future=False):
     dir_ = os.path.join(_get_staging_directory(), "scripts")
     url = "sqlite:///%s/%s" % (dir_, tempname)
-    return engines.testing_engine(url=url)
+    return testing_util.testing_engine(url=url, future=future)
 
 
-def _sqlite_testing_config(sourceless=False):
+def _sqlite_testing_config(sourceless=False, future=False):
     dir_ = os.path.join(_get_staging_directory(), "scripts")
     url = "sqlite:///%s/foo.db" % dir_
 
+    sqlalchemy_future = future or ("future" in config.db.__class__.__module__)
+
     return _write_config_file(
         """
 [alembic]
 script_location = %s
 sqlalchemy.url = %s
 sourceless = %s
+%s
 
 [loggers]
-keys = root
+keys = root,sqlalchemy
 
 [handlers]
 keys = console
@@ -115,6 +125,11 @@ level = WARN
 handlers = console
 qualname =
 
+[logger_sqlalchemy]
+level = DEBUG
+handlers =
+qualname = sqlalchemy.engine
+
 [handler_console]
 class = StreamHandler
 args = (sys.stderr,)
@@ -128,12 +143,19 @@ keys = generic
 format = %%(levelname)-5.5s [%%(name)s] %%(message)s
 datefmt = %%H:%%M:%%S
     """
-        % (dir_, url, "true" if sourceless else "false")
+        % (
+            dir_,
+            url,
+            "true" if sourceless else "false",
+            "sqlalchemy.future = true" if sqlalchemy_future else "",
+        )
     )
 
 
 def _multi_dir_testing_config(sourceless=False, extra_version_location=""):
     dir_ = os.path.join(_get_staging_directory(), "scripts")
+    sqlalchemy_future = "future" in config.db.__class__.__module__
+
     url = "sqlite:///%s/foo.db" % dir_
 
     return _write_config_file(
@@ -141,6 +163,7 @@ def _multi_dir_testing_config(sourceless=False, extra_version_location=""):
 [alembic]
 script_location = %s
 sqlalchemy.url = %s
+sqlalchemy.future = %s
 sourceless = %s
 version_locations = %%(here)s/model1/ %%(here)s/model2/ %%(here)s/model3/ %s
 
@@ -171,6 +194,7 @@ datefmt = %%H:%%M:%%S
         % (
             dir_,
             url,
+            "true" if sqlalchemy_future else "false",
             "true" if sourceless else "false",
             extra_version_location,
         )
@@ -290,7 +314,7 @@ def three_rev_fixture(cfg):
     c = util.rev_id()
 
     script = ScriptDirectory.from_config(cfg)
-    script.generate_revision(a, "revision a", refresh=True)
+    script.generate_revision(a, "revision a", refresh=True, head="base")
     write_script(
         script,
         a,
@@ -313,7 +337,7 @@ def downgrade():
         % a,
     )
 
-    script.generate_revision(b, "revision b", refresh=True)
+    script.generate_revision(b, "revision b", refresh=True, head=a)
     write_script(
         script,
         b,
@@ -338,7 +362,7 @@ def downgrade():
         encoding="utf-8",
     )
 
-    script.generate_revision(c, "revision c", refresh=True)
+    script.generate_revision(c, "revision c", refresh=True, head=b)
     write_script(
         script,
         c,
@@ -366,6 +390,9 @@ def downgrade():
 def multi_heads_fixture(cfg, a, b, c):
     """Create a multiple head fixture from the three-revs fixture"""
 
+    # a->b->c
+    #     -> d -> e
+    #     -> f
     d = util.rev_id()
     e = util.rev_id()
     f = util.rev_id()
@@ -454,6 +481,8 @@ def _multidb_testing_config(engines):
 
     dir_ = os.path.join(_get_staging_directory(), "scripts")
 
+    sqlalchemy_future = "future" in config.db.__class__.__module__
+
     databases = ", ".join(engines.keys())
     engines = "\n\n".join(
         "[%s]\n" "sqlalchemy.url = %s" % (key, value.url)
@@ -465,7 +494,7 @@ def _multidb_testing_config(engines):
 [alembic]
 script_location = %s
 sourceless = false
-
+sqlalchemy.future = %s
 databases = %s
 
 %s
@@ -493,5 +522,5 @@ keys = generic
 format = %%(levelname)-5.5s [%%(name)s] %%(message)s
 datefmt = %%H:%%M:%%S
     """
-        % (dir_, databases, engines)
+        % (dir_, "true" if sqlalchemy_future else "false", databases, engines)
     )
